#!/bin/bash
# ORAC program: Do all SMD tests sequentially 
# script has been tested on Linux platforms only. 
if [ "$1" == "" ];
then echo " TARGET program not specified; using Default"  ;
   echo "Syntax: run_basic_test.bash [TARGET]"
   echo "NB. if TARGET is not in the path, full path must be specified in TARGET"
   exit
else TARGET=$1; 
fi
if `test -e $TARGET`; 
  then  $TARGET < 1a.in >& tmp ;
  echo "Doing first test for SMD ....." 
  # First test 
  # Test on driven stretching, bending and torsion of decaalanine
  echo "Testing 1a.in..." > OUT_TEST
  grep "Total    = " tmp >> OUT_TEST; rm tmp; tail -2 WRK.ala10 >> OUT_TEST
  
  echo "Testing 1b.in..." >> OUT_TEST
  $TARGET < 1b.in >& tmp ;
  grep "Total    = " tmp >> OUT_TEST; rm tmp; tail -2 WRK.ala10 >> OUT_TEST
  
  echo "Testing 1c.in..." >> OUT_TEST
  $TARGET < 1c.in >& tmp ;
  grep "Total    = " tmp >> OUT_TEST; rm tmp; tail -2 WRK.ala10 >> OUT_TEST
  
  echo "Testing 1d.in..." >> OUT_TEST
  $TARGET < 1d.in >& tmp ;
  grep "Total    = " tmp >> OUT_TEST; rm tmp; tail -4 WRK.ala10 >> OUT_TEST
  # Second  test 
  # generate state A and B starting equilibrium distributions
  # state A phase space points (folded 10ala) in RESTART_A directory
  # state B phase space points (unfolded 10ala) in RESTART_B directory
  echo "Doing second  test for SMD ....." 
  echo "Testing 2a.in..." >> OUT_TEST
  bin/do_restart_a.bash > tmp1a  & $TARGET < 2a.in >& tmpa ;  
  grep "Total    = " tmpa >> OUT_TEST ;
  
  echo "Testing 2b.in..." >> OUT_TEST ;
  bin/do_restart_b.bash > tmp1b & $TARGET < 2b.in >& tmpb   ;
  grep "Total    = " tmpb >> OUT_TEST

  # Third test 
  #  

  echo "Doing third and final test for SMD ....." 
# generates forward works (see ancillary scripts "forward_trajectories" in bin dir 
  echo "Doing forward nonequilibrium experiments"  >> OUT_TEST
  echo "  Doing forward nonequilibrium experiments" 
  ls RESTART_A/ | wc | awk '{print $1}'   > works_forward
  bin/forward_trajectories.bash  $TARGET >> OUT_TEST
  for i in `ls FORWARD/*.wrk` ; do tail -1 $i | awk '{print $7}' >> works_forward ; done

# generates reverse works (see ancillary scripts "reverse_trajectories" in bin dir 
  echo "Doing reverse  nonequilibrium experiments"  >> OUT_TEST
  echo "  Doing reverse  nonequilibrium experiments" 
  ls RESTART_B/ | wc | awk '{print $1}'   > works_reverse
  bin/reverse_trajectories.bash  $TARGET  >> OUT_TEST
  for i in `ls REVERSE/*.wrk` ; do tail -1 $i | awk '{print $7}' >> works_reverse ; done

# compile Bennett program 
  make -C bin -f makefile 

# Launch Bennet and print out Free energy 
  bin/bennett >> OUT_TEST

# Now check if tests are OK (or almost OK) 
  diff OUT_TEST OUT >& x.0;  rm  *.out fuffa tmp* energie *.rst* *.works* *.pdb  */*.rst  *.rst FORWARD/* REVERSE/* *prmtpg WR* wr* check* bin/bennett >& /dev/null

  if `test -s x.0`; 
  then rm x.0;  
       echo "TEST FAILED! Could be OK anyway; Differences may be insignificant" 
       echo "Compare OUT and OUT_TEST and/or check all tests one by one using the Makefile"; exit  
  else echo "***** Test OK !!! ****** "; rm OUT_TEST *.0* ; exit ; 
  fi
else 
   echo "No program to run: compile orac or specify existing executable  " 
fi
