      subroutine init_rem(iproc,nproc,kprint,ntap,ltors,int14p,ltor
     &     ,int14,remltor,remint14,rematom)
      implicit none
      integer iproc,nproc,kprint,ntap,ltors,int14p
      integer ltor(4,*),int14(2,*)
      logical remltor(*),remint14(*),rematom(*)
      integer i,i1,j
      integer nerrors,ierr
      real*8 dum1,dum2,dum3
      real*8 buffer1(0:100),buffer2(0:100),buffer3(0:100)
      CHARACTER*27 err_unr(4)
      CHARACTER*15 err_fnf
      character*80 errmsg
      logical exist

#ifdef PARALLEL

      include 'mpif.h'
      include 'crem.inc'

      err_unr(1)='Unrecognized command  ---> '
      err_unr(2)='Unrecognized subcommand -> '
      err_unr(3)='Unrecognized keyword ----> '
      err_unr(4)='UNSUPPORTED  COMMAND ----> '
      err_fnf = ' file not found' 
      nerrors = 0
      
      
! cdist = 0: restart from an old run
! cdist = 1: new run with a geometric progression for temperatures 
! cdist = 2: new run with a custom progression 

      if(cdist.eq.0) then      

! ****************************************************************************************
! ********************************* restart an old run ***********************************
! ****************************************************************************************

! look for REM.set; then restart
        inquire(FILE="REM.set",EXIST=exist)
        if(exist) then
          open(unit=999,file="REM.set")
          do i = 1,nproc
            read(999,*) dum1, dum2, dum3
!            write(*,*) 'dum', dum1,dum2,dum3
!            write(*,*) 'para_index', para_index
            if(para_index.eq.i) then 
              rem_factor(1) = dum1
              rem_factor(2) = dum2
              rem_factor(3) = dum3
            endif
          enddo
          close(999)
          write(*,8099) para_index,(rem_factor(i),i=1,3)
8099      FORMAT( "-- FROM RESTART FILE:", /, "-- REPLICA INDEX"
     &         ,I10,/,"-- SCALING_FACTOR",3F10.4/) 
        else
          errmsg='Cannot find REM.set for restart'
          CALL xerror(errmsg,80,1,30)
          nerrors=nerrors+1
        endif

      else                    

! ****************************************************************************************
! *********************************  run from scratch  ***********************************
! ****************************************************************************************

        para_index = iproc + 1 ! assign to each process a replica index and a scaling 
        
        if(cdist.eq.1) then     ! use a geometric progression
          
          do i = 1,3
            if(nproc.gt.1) then 
              rem_factor(i) = rem_factor_max(i)**(dfloat(iproc)
     &             /(nproc -1.d0))
            else                ! if the REM run has only one replica, the scaling factors are the final ones
              rem_factor(i) = rem_factor_max(i)
            endif
          enddo
          
        elseif(cdist.eq.2) then ! read scaling factors from a file
          
          INQUIRE(FILE="../REM.set",EXIST=exist)
          IF(exist) THEN
            write(*,*) 'Found a REM.set file'
            OPEN(unit=999,file='../REM.set')
            do i = 0,nproc-1
              read(999,*) dum1, dum2, dum3
              if(i.eq.iproc) then 
                rem_factor(1) = dum1
                rem_factor(2) = dum2
                rem_factor(3) = dum3
              endif
            enddo
            CLOSE(999)
          ELSE
            nerrors = nerrors + 1
            errmsg = 'REM.set' // err_fnf 
            CALL xerror(errmsg,80,1,30)
          ENDIF

        else
          
          nerrors = nerrors + 1
          errmsg = err_unr(3) //
     &         'in REM SETUP: must be 0, 1 or 2'
          CALL xerror(errmsg,80,1,30)
          
        endif

      endif
      
! if errors were found, stop
      if(nerrors.gt.0) then 
        errmsg= ' ERRORS WHILE VERIFYING REM INPUT' 
        call xerror(errmsg,80,1,2)
        stop
      else
        WRITE(kprint,'(5x,a)') 'REM INPUT OK!!'
      endif
      
      CALL MPI_Barrier(MPI_COMM_WORLD,ierr) 
      
! ****************************************************************************************
! ****************  if all is ok, exchange informations among replicas *******************
! **************************************************************************************** 
      
      call MPI_ALLGATHER(para_index,1,MPI_INTEGER,rem_table,1 ! build a table for the replica indexes
     &     ,MPI_INTEGER,MPI_COMM_WORLD,ierr)
      
      call MPI_ALLGATHER(rem_factor(1),1,MPI_DOUBLE_PRECISION,buffer1 ! the root process gather the scaling factors from all the replicas
     &     ,1,MPI_DOUBLE_PRECISION,MPI_COMM_WORLD,ierr)
      call MPI_ALLGATHER(rem_factor(2),1,MPI_DOUBLE_PRECISION,buffer2 ! the root process gather the scaling factors from all the replicas
     &     ,1,MPI_DOUBLE_PRECISION,MPI_COMM_WORLD,ierr)
      call MPI_ALLGATHER(rem_factor(3),1,MPI_DOUBLE_PRECISION,buffer3 ! the root process gather the scaling factors from all the replicas
     &     ,1,MPI_DOUBLE_PRECISION,MPI_COMM_WORLD,ierr)
      
! build rem_mat
      do i = 0,nproc-1
        rem_mat(rem_table(i),1) = buffer1(i)
        rem_mat(rem_table(i),2) = buffer2(i)
        rem_mat(rem_table(i),3) = buffer3(i)
      enddo
      if(cdist .GT. 0) then 
        OPEN(unit=999,file="REM.set")
        do i=1,nproc
          write(999,'(3f12.5)') rem_mat(i,1), rem_mat(i,2), rem_mat(i
     &         ,3)
        enddo
        CLOSE(999)
      endif

! ****************************************************************************************
! **************************  prepare a solute tempering run  ****************************
! ****************************************************************************************

      if(rem_segment) then 
! build three logical arrays, remltor and remint14 and rematom
        do i = 1,int14p
          remint14(i) = .FALSE.
        enddo
        do i = 1,ltors
          remltor(i) = .FALSE.
          do j = 1,rem_groups
            do i1 = 1,4
! if the atoms of the torsion overlap with the subsets of atoms, then 
              if((ltor(i1,i).GE.rem_group(1,j)).AND.
     &             (ltor(i1,i).LE.rem_group(2,j))) remltor(i) = .TRUE.
            enddo
          enddo
          if(remltor(i)) then 
! the 14 interaction will be scaled too
! find the corresponding 14 interaction
            do j = 1,int14p
              if((int14(1,j).EQ.ltor(1,i)).AND.(int14(2,j).EQ.ltor(4
     &             ,i))) then
                remint14(j) = .TRUE.
              endif
            enddo
          endif
        enddo
        do i = 1,ntap
          rematom(i) = .false.
          do j = 1,rem_groups
            if((i.GE.rem_group(1,j)).AND.(i.LE.rem_group(2,j))) 
     &           rematom(i) = .TRUE.
          enddo
        enddo
      endif


      step_rem = 0

#endif

      return 
      end
      
